@@ -1,4 +1,4 @@
-class Registry(object):
+class Registry:
     def __init__(self):
         self._registry = {}
         self._field_registry = {}
