@@ -2,7 +2,7 @@ import json
 
 import pytest
 
-from mock import patch
+from unittest.mock import patch
 
 from django.db import connection
 
@@ -107,11 +107,11 @@ def test_reports_validation_errors(client):
     assert response_json(response) == {
         "errors": [
             {
-                "message": 'Cannot query field "unknownOne" on type "QueryRoot".',
+                "message": "Cannot query field 'unknownOne' on type 'QueryRoot'.",
                 "locations": [{"line": 1, "column": 9}],
             },
             {
-                "message": 'Cannot query field "unknownTwo" on type "QueryRoot".',
+                "message": "Cannot query field 'unknownTwo' on type 'QueryRoot'.",
                 "locations": [{"line": 1, "column": 21}],
             },
         ]
@@ -132,7 +132,7 @@ def test_errors_when_missing_operation_name(client):
     assert response_json(response) == {
         "errors": [
             {
-                "message": "Must provide operation name if query contains multiple operations."
+                "message": "Must provide operation name if query contains multiple operations.",
             }
         ]
     }
@@ -472,8 +472,7 @@ def test_handles_syntax_errors_caught_by_graphql(client):
         "errors": [
             {
                 "locations": [{"column": 1, "line": 1}],
-                "message": "Syntax Error GraphQL (1:1) "
-                'Unexpected Name "syntaxerror"\n\n1: syntaxerror\n   ^\n',
+                "message": "Syntax Error: Unexpected Name 'syntaxerror'.",
             }
         ]
     }
@@ -508,7 +507,7 @@ def test_handles_invalid_json_bodies(client):
 
 def test_handles_django_request_error(client, monkeypatch):
     def mocked_read(*args):
-        raise IOError("foo-bar")
+        raise OSError("foo-bar")
 
     monkeypatch.setattr("django.http.request.HttpRequest.read", mocked_read)
 
